use based::auth::MaybeUser;
use based::request::{RawResponse, RequestContext, StringResponse, respond_with};
use based::ui::components::Shell;
use based::ui::{prelude::*, render_page};
use maud::{PreEscaped, html};
use pacco::pkg::mirror::MirrorRepository;
use rocket::http::{ContentType, Status};
use rocket::{State, get};

use pacco::pkg::Repository;
use pacco::pkg::arch::Architecture;

use crate::config::Config;

pub mod push;
pub mod ui;
pub mod user;

#[get("/")]
pub async fn index_page(
    ctx: RequestContext,
    user: MaybeUser,
    config: &State<Config>,
) -> StringResponse {
    let repos: Vec<String> = Repository::list();

    let content = html!(
        (Flex(html! {
            h1 class="text-4xl font-bold pb-6" { "Repositories" };
           @if let Some(user) = user.take_user() {
               a class="text-lg" href="/account" { (user.username) };
           };
        }).justify(Justify::Between))

        div class="grid grid-cols-1 sm:grid-cols-2 lg:grid-cols-3 gap-6" {
            @for repo in repos {
                // TODO : Implement class transition
                (Link(&format!("/{repo}"),
                    Rounded(
                        Padding(
                        Shadow::medium(
                            Flex(Hover(
                                Background(Gray::_600, Nothing()),
                                Background(Gray::_700,
                                    Div().vanish().push(Text(&repo).medium().color(&Gray::_100))
                                    .push(html! {
                                        @if config.is_mirrored_repo(&repo) {
                                            (Padding(Rounded(Background(Blue::_500, Text("Mirrored").white().medium().sm())).size(Size::Full)).x(ScreenValue::_3).y(ScreenValue::_1))
                                        };
                                    })
                            ))).items_center().gap(4))).all(ScreenValue::_4))
                            .size(Size::Large)))
            }
        }
    );

    render(content, "Repositories", ctx).await
}

#[get("/pkg/<repo>/<arch>/<pkg_name>")]
pub async fn pkg_route(
    repo: &str,
    arch: &str,
    pkg_name: &str,
    config: &State<Config>,
) -> RawResponse {
    let arch = Architecture::parse(arch).unwrap();

    if config.is_mirrored_repo(repo) {
        let repo = MirrorRepository::new(repo);
        if is_repo_db(pkg_name) {
            if pkg_name.ends_with("sig") {
                return respond_with(
                    Status::Ok,
                    ContentType::new("application", "pgp-signature"),
                    repo.sig_content(arch, config.mirrorlist().unwrap())
                        .await
                        .unwrap(),
                );
            } else {
                return respond_with(
                    Status::Ok,
                    ContentType::new("application", "tar"),
                    repo.db_content(arch, config.mirrorlist().unwrap())
                        .await
                        .unwrap(),
                );
            }
        }

        let pkg = repo
            .get_pkg(pkg_name, config.mirrorlist().unwrap())
            .await
            .unwrap();

        if pkg_name.ends_with("pkg.tar.zst") {
            return respond_with(
                Status::Ok,
                ContentType::new("application", "tar"),
                pkg.pkg_content().unwrap(),
            );
        } else if pkg_name.ends_with("pkg.tar.zst.sig") {
            return respond_with(
                Status::Ok,
                ContentType::new("application", "pgp-signature"),
                pkg.sig_content().unwrap(),
            );
        }

        return respond_with(
            Status::Ok,
            ContentType::Plain,
            "Not found".as_bytes().to_vec(),
        );
    }

    if let Some(repo) = Repository::new(repo) {
        if is_repo_db(pkg_name) {
            if pkg_name.ends_with("sig") {
                return respond_with(
                    Status::Ok,
                    ContentType::new("application", "pgp-signature"),
                    repo.sig_content(arch).unwrap(),
                );
            } else {
                return respond_with(
                    Status::Ok,
                    ContentType::new("application", "tar"),
                    repo.db_content(arch).unwrap(),
                );
            }
        }

        let pkg = repo.get_pkg(pkg_name).unwrap();

        if pkg_name.ends_with("pkg.tar.zst") {
            return respond_with(
                Status::Ok,
                ContentType::new("application", "tar"),
                pkg.pkg_content().unwrap(),
            );
        } else if pkg_name.ends_with("pkg.tar.zst.sig") {
            return respond_with(
                Status::Ok,
                ContentType::new("application", "pgp-signature"),
                pkg.sig_content().unwrap(),
            );
        }
    }

    respond_with(
        Status::Ok,
        ContentType::Plain,
        "Not found".as_bytes().to_vec(),
    )
}

pub async fn render(
    content: PreEscaped<String>,
    title: &str,
    ctx: RequestContext,
) -> StringResponse {
    render_page(
        content,
        title,
        ctx,
        &Shell::new(
            html! {
                script src="https://cdn.tailwindcss.com" {};
                script src="/assets/htmx.min.js" {};
                meta name="viewport" content="width=device-width, initial-scale=1.0";
            },
            html! {},
            Some("bg-gray-900 text-gray-200 min-h-screen p-10".to_string()),
        ),
    )
    .await
}

pub fn is_repo_db(pkg_name: &str) -> bool {
    pkg_name.ends_with("db.tar.gz")
        || pkg_name.ends_with("db")
        || pkg_name.ends_with("db.tar.gz.sig")
        || pkg_name.ends_with("db.sig")
}
